@@ -103,15 +103,10 @@ try_str_to_guint16 (gchar    *start,
   g_return_val_if_fail (end != NULL, FALSE);
   g_return_val_if_fail (result != NULL, FALSE);
 
+  errno = 0;
   *result = strtol (start, end, 10);
 
-  switch (errno) {
-  case EINVAL:
-  case ERANGE:
-    return FALSE;
-  default:
-    return TRUE;
-  }
+  return errno == 0;
 }
 
 static gboolean
@@ -168,6 +163,8 @@ parse_mapping_invert (gchar     *start,
 
     return TRUE;
   default:
+    *invert = FALSE;
+
     return TRUE;
   }
 }
@@ -426,7 +423,7 @@ set_from_mapping_string (ManetteMapping *self,
   gchar **splitted_mapping;
   gchar *destination_string;
   gchar *source_string;
-  ManetteMappingBinding binding;
+  ManetteMappingBinding binding = {};
 
   mappings = g_strsplit (mapping_string, ",", 0);
   mappings_length = g_strv_length (mappings);
@@ -443,6 +440,13 @@ set_from_mapping_string (ManetteMapping *self,
     destination_string = splitted_mapping[0];
     source_string = splitted_mapping[1];
 
+    /* Skip the platform key. */
+    if (g_strcmp0 ("platform", splitted_mapping[0]) == 0) {
+      g_strfreev (splitted_mapping);
+
+      continue;
+    }
+
     if (!parse_mapping_destination (destination_string, &binding)) {
       g_critical ("Invalid mapping destination: %s", splitted_mapping[0]);
       g_strfreev (splitted_mapping);
@@ -451,9 +455,7 @@ set_from_mapping_string (ManetteMapping *self,
     }
 
     if  (binding.destination.type == EV_MAX) {
-      if (g_strcmp0 (destination_string, "platform") != 0)
-        g_debug ("Invalid token: %s", destination_string);
-
+      g_debug ("Invalid token: %s", destination_string);
       g_strfreev (splitted_mapping);
 
       continue;
